@@ -7,6 +7,7 @@ from weakref import WeakKeyDictionary
 
 
 PY2 = sys.version_info[0] == 2
+DEFAULT_COLUMNS = 80
 
 
 _ansi_re = re.compile('\033\[((?:\d|;)*)([a-zA-Z])')
@@ -447,12 +448,16 @@ class _AtomicFile(object):
 
 auto_wrap_for_ansi = None
 colorama = None
+get_winterm_size = None
 
 
 # If we're on Windows, we provide transparent integration through
 # colorama.  This will make ANSI colors through the echo function
 # work automatically.
 if sys.platform.startswith('win'):
+    # Windows has a smaller terminal
+    DEFAULT_COLUMNS = 79
+
     try:
         import colorama
     except ImportError:
@@ -461,6 +466,11 @@ if sys.platform.startswith('win'):
         _ansi_stream_wrappers = WeakKeyDictionary()
 
         def auto_wrap_for_ansi(stream):
+            """This function wraps a stream so that calls through colorama
+            are issued to the win32 console API to recolor on demand.  It
+            also ensures to reset the colors if a write call is interrupted
+            to not destroy the console afterwards.
+            """
             try:
                 cached = _ansi_stream_wrappers.get(stream)
             except Exception:
@@ -468,13 +478,30 @@ if sys.platform.startswith('win'):
             if cached is not None:
                 return cached
             strip = not isatty(stream)
-            rv = colorama.AnsiToWin32(stream, strip=strip).stream
+            ansi_wrapper = colorama.AnsiToWin32(stream, strip=strip)
+            rv = ansi_wrapper.stream
+            _write = rv.write
+
+            def _safe_write(s):
+                try:
+                    return _write(s)
+                except:
+                    ansi_wrapper.reset_all()
+                    raise
+
+            rv.write = _safe_write
             try:
                 _ansi_stream_wrappers[stream] = rv
             except Exception:
                 pass
             return rv
 
+        def get_winterm_size():
+            win = colorama.win32.GetConsoleScreenBufferInfo(
+                colorama.win32.STDOUT).srWindow
+            return win.Right - win.Left, win.Bottom - win.Top
+
+
 
 def strip_ansi(value):
     return _ansi_re.sub('', value)
